#include <pspdisplay.h>
#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspgu.h>
#include <png.h>
#include <stdio.h>
#include "graphics.h"

#define printf pspDebugScreenPrintf
#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))
#define RGB(r, g, b) (0xFF000000 | ((b)<<16) | ((g)<<8) | (r))


PSP_MODULE_INFO("Move Me", 0, 1, 1);

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
          sceKernelExitGame();
          return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
          int cbid;

          cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
          sceKernelRegisterExitCallback(cbid);

          sceKernelSleepThreadCB();

          return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
          int thid = 0;

          thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
          if(thid >= 0) {
                    sceKernelStartThread(thid, 0, 0);
          }

          return thid;
}

int main() {
	char buffer[200];
	Image* ourChar;

	pspDebugScreenInit();
	SetupCallbacks();
	initGraphics();

	sprintf(buffer,"arrow.png");
	ourChar = loadImage(buffer);
	if (!ourChar) {
		//Image load failed
		printf("Image load failed!\n");
	} else {
		//Image load worked
		//printf("Image load worked!\n");
		int x=0,y=0;
		SceCtrlData pad;

		int topBound    = 0;
		int bottomBound = 272;
		int leftBound   = 0;
		int rightBound  = 480;

		int imageSize = 32;
		int dir = 0;
		int go = 0;
		int speed = 10;
		int trail = 0;

		int locs[4];

		int i;
		for(i=0; i<4; i++) {
			locs[i] = imageSize * i;
		}

		while(1) {
			sceCtrlReadBufferPositive(&pad, 1);
			if(pad.Buttons & PSP_CTRL_UP) {
				dir = 0;
			} else if(pad.Buttons & PSP_CTRL_RIGHT) {
				dir = 1;
			} else if(pad.Buttons & PSP_CTRL_DOWN) {
				dir = 2;
			} else if(pad.Buttons & PSP_CTRL_LEFT) {
				dir = 3;
			}

			if(pad.Buttons & PSP_CTRL_TRIANGLE) {
				trail = !trail;
				for(i=0; i<20; i++) {
                    sceDisplayWaitVblankStart();
				}
			}

			if(pad.Buttons & PSP_CTRL_LTRIGGER) {
				if(speed > 1) {
					speed--;
					for(i=0; i<2; i++) {
						sceDisplayWaitVblankStart();
					}
				}
			}
			if(pad.Buttons & PSP_CTRL_RTRIGGER) {
				if(speed < 20) {
					speed++;
					for(i=0; i<2; i++) {
						sceDisplayWaitVblankStart();
					}
				}
			}

			if(pad.Buttons & PSP_CTRL_CROSS) {
				//printf("Go");
				go = 1;
			} else {
				//printf("Don't Go");
				go = 0;
			}

			if(go) {
				switch(dir) {
					case 0:
						if(y >= topBound + speed) {
							y -= speed;
						}
						break;
					case 1:
						if(x <= rightBound - speed - imageSize) {
							x += speed;
						}
						break;
					case 2:
						if(y <= bottomBound - speed - imageSize) {
							y += speed;
						}
						break;
					case 3:
						if(x >= leftBound + speed) {
							x -= speed;
						}
						break;
					default:
						//Shouldn't Ever Run
						printf("ERROR: DIRECTION OUTSIDE OF RANGE");
						while(1) {
							sceDisplayWaitVblankStart();
						}
						break;
				}
			}
			
			if(!trail) {
				clearScreen(RGB(0, 0, 0));
			}
			blitAlphaImageToScreen(locs[dir] , 0, imageSize, imageSize, ourChar, x, y);
			flipScreen();

			/*for(i=0; i<5; i++) {
                    sceDisplayWaitVblankStart();
             }*/
		}
	}

	sceKernelSleepThread();
	return 0;
}
